#include "filebrowser.h"
#include "fileiconview.h"

#include <qprogressbar.h>
#include <qlabel.h>
#include <qstatusbar.h>
#include <qcombobox.h>
#include <qpixmap.h>
#include <qtoolbutton.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qlayout.h>

static const char* cdtoparent_xpm[]={
    "15 13 3 1",
    ". c None",
    "* c #000000",
    "a c #ffff99",
    "..*****........",
    ".*aaaaa*.......",
    "***************",
    "*aaaaaaaaaaaaa*",
    "*aaaa*aaaaaaaa*",
    "*aaa***aaaaaaa*",
    "*aa*****aaaaaa*",
    "*aaaa*aaaaaaaa*",
    "*aaaa*aaaaaaaa*",
    "*aaaa******aaa*",
    "*aaaaaaaaaaaaa*",
    "*aaaaaaaaaaaaa*",
    "***************"};

static const char* newfolder_xpm[] = {
    "15 14 4 1",
    " 	c None",
    ".	c #000000",
    "+	c #FFFF00",
    "@	c #FFFFFF",
    "          .    ",
    "               ",
    "          .    ",
    "       .     . ",
    "  ....  . . .  ",
    " .+@+@.  . .   ",
    "..........  . .",
    ".@+@+@+@+@..   ",
    ".+@+@+@+@+. .  ",
    ".@+@+@+@+@.  . ",
    ".+@+@+@+@+.    ",
    ".@+@+@+@+@.    ",
    ".+@+@+@+@+.    ",
    "...........    "};

FileBrowser::FileBrowser( QWidget* parent, const char* name, WFlags f)
    : QWidget( parent, name, f) 
{
    QVBoxLayout* vbox = new QVBoxLayout(this, 2, 2);

    // toolbar

    QHBoxLayout* hbox = new QHBoxLayout(2);
    vbox->addLayout( hbox, AlignRight );

    QLabel* path = new QLabel( tr( " Path: " ), this );
    hbox->addWidget( path );

    pathCombo = new QComboBox( TRUE, this );
    pathCombo->setAutoCompletion( TRUE );
    connect( pathCombo, SIGNAL( activated( const QString & ) ),
	     this, SLOT ( changePath( const QString & ) ) );
    hbox->addWidget(pathCombo, 1);

    QPixmap pix;

    pix = QPixmap( cdtoparent_xpm );
    upButton = new QToolButton( this, "up");
    upButton->setPixmap( pix );
    connect (upButton, SIGNAL( clicked()), 
	     this, SLOT( cdUp()) );
    hbox->addWidget(upButton);

    pix = QPixmap( newfolder_xpm );
    mkdirButton = new QToolButton( this, "mkdir");
    mkdirButton->setPixmap( pix );
    connect (mkdirButton, SIGNAL( clicked()), 
	     this, SLOT( newFolder()) );
    hbox->addWidget(mkdirButton);

    // fileview 

    fileview = new FileIconView( QDir::current().path(), this );
    fileview->setSelectionMode( QIconView::Extended );
    connect( fileview, SIGNAL( enableUp( const bool ) ),
	     this, SLOT( enableUp( const bool) ) );
    connect( fileview, SIGNAL( enableMkdir( const bool ) ),
	     this, SLOT( enableMkdir( const bool ) ) );
    connect( fileview, SIGNAL( directoryChanged( const QString & ) ),
	     this, SLOT( directoryChanged( const QString & ) ) );
    connect( fileview, SIGNAL( startReadDir( int ) ),
	     this, SLOT( startReadDir( int ) ) );
    connect( fileview, SIGNAL( readNextDir() ),
	     this, SLOT( readNextDir() ) );
    connect( fileview, SIGNAL( readDirDone() ),
	     this, SLOT( readDirDone() ) );
    connect( fileview, SIGNAL( runCommand( const QString& ) ),
	     this, SLOT( runCommand( const QString& ) ) );
    vbox->addWidget( fileview );


    // statusbar 

    QStatusBar* statusBar = new QStatusBar( this );
    vbox->addWidget( statusBar );

    label = new QLabel( statusBar );
    statusBar->addWidget( label, 2, TRUE );
    progress = new QProgressBar( statusBar );
    statusBar->addWidget( progress, 1, TRUE );

    // process

    process = new QProcess(this);
    connect( process, SIGNAL( processExited() ),
	     this, SLOT( processExited() ) );
}

void FileBrowser::setPathCombo(const QString& dir)
{
    int i = 0;
    bool found = FALSE;
    for ( i = 0; i < pathCombo->count(); ++i ) {
	if ( pathCombo->text( i ) == dir) {
	    found = TRUE;
	    break;
	}
    }

    if ( found )
	pathCombo->setCurrentItem( i );
    else {
	pathCombo->insertItem( dir );
	pathCombo->setCurrentItem( pathCombo->count() - 1 );
    }

}

void FileBrowser::directoryChanged( const QString &dir )
{
    setCaption( dir );
    setPathCombo( dir );
}

void FileBrowser::startReadDir( int dirs )
{
    label->setText( tr( " Reading Directory..." ) );
    progress->reset();
    progress->setTotalSteps( dirs );
}

void FileBrowser::readNextDir()
{
    int p = progress->progress();
    progress->setProgress( ++p );
}

void FileBrowser::readDirDone()
{
    label->setText( tr( " Reading Directory Done." ) );
    progress->setProgress( progress->totalSteps() );
}

void FileBrowser::cdUp()
{
    QDir dir = fileview->currentDir();
    dir.cd( ".." );
    fileview->setDirectory( dir );
}

void FileBrowser::newFolder()
{
    fileview->newDirectory();
}

void FileBrowser::changePath( const QString &path )
{
    if ( QFileInfo( path ).exists() )
	fileview->setDirectory( path );
    else
	setPathCombo( caption() );
}

void FileBrowser::enableUp( const bool value )
{
    upButton->setEnabled( value );
}

void FileBrowser::enableMkdir( const bool value )
{
    mkdirButton->setEnabled( value );
}

void FileBrowser::runCommand(const QString& command) {
    if ( process->isRunning() )
        // ignore 
        return;
    
    process->clearArguments();
    process->setArguments(QStringList::split(' ', command));

    if ( !process->start() ) 
      QMessageBox::critical(this, "Run system command",
			    QString("system command\n%1\nfails!").arg(command),
			    "Ok");
    else 
      label->setText(" Running...");
}

void FileBrowser::processExited() {
    // reload
    QDir current = fileview->currentDir();
    fileview->setDirectory( current );
}
