#include "fileiconview.h"

#include <qpainter.h>
#include <qstringlist.h>
#include <qpixmap.h>
#include <qmime.h>
#include <qstrlist.h>
#include <qdragobject.h>
#include <qmessagebox.h>
#include <qevent.h>
#include <qpopupmenu.h>
#include <qcursor.h>

#include <qmainwindow.h>

#include "pixmap.h"

/*****************************************************************************
 *
 * Class FileIconDrag
 *
 *****************************************************************************/

FileIconDrag::FileIconDrag( QWidget * dragSource, const char* name )
    : QIconDrag( dragSource, name )
{
}

const char* FileIconDrag::format( int i ) const
{
    if ( i == 0 )
	return "application/x-qiconlist";
    else if ( i == 1 )
	return "text/uri-list";
    else
	return 0;
}

QByteArray FileIconDrag::encodedData( const char* mime ) const
{
    QByteArray a;
    if ( QString( mime ) == "application/x-qiconlist" ) {
	a = QIconDrag::encodedData( mime );
    } else if ( QString( mime ) == "text/uri-list" ) {
	QString s = urls.join( "\r\n" );
	a.resize( s.length() );
	memcpy( a.data(), s.latin1(), s.length() );
    }
    return a;
}

bool FileIconDrag::canDecode( QMimeSource* e )
{
    return e->provides( "application/x-qiconlist" ) ||
	e->provides( "text/uri-list" );
}

void FileIconDrag::append( const QIconDragItem &item, const QRect &pr,
			     const QRect &tr, const QString &url )
{
    QIconDrag::append( item, pr, tr );
    urls << url;
}

/*****************************************************************************
 *
 * Class FileIconViewItem
 *
 *****************************************************************************/

FileIconViewItem::FileIconViewItem( FileIconView *parent, QFileInfo *fi )
    : QIconViewItem( parent, fi->fileName() ), itemFileName( fi->filePath() ),
      itemFileInfo( fi ), checkSetText( FALSE )
{
    if ( itemFileInfo->isDir() )
	itemType = Dir;
    else if ( itemFileInfo->isFile() )
	itemType = File;
    if ( itemFileInfo->isSymLink() )
	itemType = Link;

    setDropEnabled( itemType == Dir && QDir( itemFileName ).isReadable() ||
		    itemType == File && itemFileInfo->isWritable() );
    calcRect();
    
    if ( itemFileInfo->fileName() == "." ||
	 itemFileInfo->fileName() == ".." )
	setRenameEnabled( FALSE );

    checkSetText = TRUE;

    QObject::connect( &timer, SIGNAL( timeout() ),
		      iconView(), SLOT( openFolder() ) );
}

void FileIconViewItem::paintItem( QPainter *p, const QColorGroup &cg )
{
    if ( itemFileInfo->isSymLink() ) {
	QFont f( p->font() );
	f.setItalic( TRUE );
	p->setFont( f );
    }

    QIconViewItem::paintItem( p, cg );
}

QPixmap *FileIconViewItem::pixmap() const
{
    return getPixmap( itemFileInfo );
}

FileIconViewItem::~FileIconViewItem()
{
    delete itemFileInfo;
}

void FileIconViewItem::setText( const QString &text )
{
    if ( checkSetText ) {
	if ( text == "." || text == "." || text.isEmpty() )
	    return;
	QDir dir( itemFileInfo->dir() );
	if ( dir.rename( itemFileInfo->fileName(), text ) ) {
	    itemFileName = itemFileInfo->dirPath( TRUE ) + "/" + text;
	    delete itemFileInfo;
	    itemFileInfo = new QFileInfo( itemFileName );
	    QIconViewItem::setText( text );
	}
    } else {
	QIconViewItem::setText( text );
    }
}

bool FileIconViewItem::acceptDrop( const QMimeSource *e ) const
{
    if ( type() == Dir && e->provides( "text/uri-list" ) &&
	 dropEnabled() )
        return TRUE;

    return FALSE;
}

void FileIconViewItem::dropped( QDropEvent *e, const QValueList<QIconDragItem> & )
{
    timer.stop();

    if ( !QUriDrag::canDecode( e ) ) {
	e->ignore();
	return;
    }

    QStrList filelst;
    QUriDrag::decode( e, filelst );

    if ( e->action() == QDropEvent::Copy )
      ((FileIconView*)iconView())->copy(filelst, itemFileName);
    if ( e->action() == QDropEvent::Move)
      ((FileIconView*)iconView())->move(filelst, itemFileName);

    e->acceptAction();
}

void FileIconViewItem::dragEntered()
{
    if ( type() != Dir ||
	 type() == Dir && !QDir( itemFileName ).isReadable() )
	return;

    ( (FileIconView*)iconView() )->setOpenItem( this );
    timer.start( 750 );
}

void FileIconViewItem::dragLeft()
{
    if ( type() != Dir ||
	 type() == Dir && !QDir( itemFileName ).isReadable() )
	return;

    timer.stop();
}

/*****************************************************************************
 *
 * Class FileIconView
 *
 *****************************************************************************/

FileIconView::FileIconView( const QString &dir, QWidget *parent, const char *name )
    : QIconView( parent, name ), viewDir( dir ), newFolderNum( 0 )
{
    createPixmaps();

    setGridX( 75 );
    setResizeMode( Adjust );
    setWordWrapIconText( FALSE );

    connect( this, SIGNAL( doubleClicked( QIconViewItem * ) ),
	     this, SLOT( itemDoubleClicked( QIconViewItem * ) ) );
    connect( this, SIGNAL( returnPressed( QIconViewItem * ) ),
	     this, SLOT( itemDoubleClicked( QIconViewItem * ) ) );
    connect( this, SIGNAL( dropped( QDropEvent *, const QValueList<QIconDragItem> & ) ),
	     this, SLOT( slotDropped( QDropEvent *, const QValueList<QIconDragItem> & ) ) );
    connect( this, SIGNAL( mouseButtonPressed(int, QIconViewItem*, const QPoint &) ),
	     this, SLOT( mouseButton(int, QIconViewItem*) ) );
    connect( this, SIGNAL( contextMenuRequested(QIconViewItem*, const QPoint &) ),
	     this, SLOT( contextMenu(QIconViewItem*) ) );

    setHScrollBarMode( AlwaysOff );
    setVScrollBarMode( Auto );

    setAutoArrange( TRUE );
    setSorting( TRUE );
    openItem = 0;
}

void FileIconView::openFolder()
{
    if ( !openItem )
	return;
    if ( openItem->type() != FileIconViewItem::Dir ||
	 openItem->type() == FileIconViewItem::Dir &&
	 !QDir( openItem->itemFileName ).isReadable() )
	return;

    openItem->timer.stop();
    setDirectory( openItem->itemFileName );
}

void FileIconView::setDirectory( const QString &dir )
{
    viewDir = QDir( dir );
    readDir( viewDir );
}

void FileIconView::setDirectory( const QDir &dir )
{
    viewDir = dir;
    readDir( viewDir );
}

void FileIconView::newDirectory()
{
    setAutoArrange( FALSE );
    selectAll( FALSE );
    if ( viewDir.mkdir( QString( "NewFolder%1" ).arg( ++newFolderNum ) ) ) {
	QFileInfo *fi = new QFileInfo( viewDir, QString( "NewFolder%1" ).arg( newFolderNum ) );
	FileIconViewItem *item = new FileIconViewItem( this, new QFileInfo( *fi ) );
	item->setKey( QString( "000000%1" ).arg( fi->fileName() ) );
	delete fi;
	repaintContents( contentsX(), contentsY(), contentsWidth(), contentsHeight(), FALSE );
	ensureItemVisible( item );
	item->setSelected( TRUE, TRUE );
	setCurrentItem( item );
	repaintItem( item );
	// qApp->processEvents();
	item->rename();
    }
    setAutoArrange( TRUE );
}

QDir FileIconView::currentDir()
{
    return viewDir;
}

static bool isRoot( const QString &s )
{
#if defined(Q_OS_UNIX)
    if ( s == "/" )
	return TRUE;
#elif defined(Q_OS_WIN32)
    QString p = s;
    if ( p.length() == 3 &&
	 p.right( 2 ) == ":/" )
	return TRUE;
    if ( p[ 0 ] == '/' && p[ 1 ] == '/' ) {
	int slashes = p.contains( '/' );
	if ( slashes <= 3 )
	    return TRUE;
	if ( slashes == 4 && p[ (int)p.length() - 1 ] == '/' )
	    return TRUE;
    }
#endif

    return FALSE;
}

void FileIconView::readDir( const QDir &dir )
{
    if ( !dir.isReadable() )
	return;

    emit enableUp ( isRoot( dir.absPath() ) ? FALSE : TRUE );

    clear();

    emit directoryChanged( dir.absPath() );

    const QFileInfoList *filist = dir.entryInfoList( QDir::DefaultFilter, QDir::DirsFirst | QDir::Name );

    emit startReadDir( filist->count() );

    QFileInfoListIterator it( *filist );
    QFileInfo *fi;
    bool allowRename = FALSE, allowRenameSet = FALSE;
    while ( ( fi = it.current() ) != 0 ) {
	++it;
	if ( fi && fi->fileName() == ".." && ( fi->dirPath() == "/" || fi->dirPath().isEmpty() ) )
	    continue;
	emit readNextDir();
	FileIconViewItem *item = new FileIconViewItem( this, new QFileInfo( *fi ) );
	if ( fi->isDir() )
	    item->setKey( QString( "000000%1" ).arg( fi->fileName() ) );
	else
	    item->setKey( fi->fileName() );
	if ( !allowRenameSet ) {
	    if ( !QFileInfo( fi->absFilePath() ).isWritable() ||
		 item->text() == "." || item->text() == ".." )
		allowRename = FALSE;
	    else
		allowRename = TRUE;
	    if ( item->text() == "." || item->text() == ".." )
		allowRenameSet = FALSE;
	    else
		allowRenameSet = TRUE;
	}
	item->setRenameEnabled( allowRename );
    }

    emit enableMkdir( QFileInfo( dir.absPath() ).isWritable() ? TRUE : FALSE);

    emit readDirDone();
}

void FileIconView::itemDoubleClicked( QIconViewItem *i )
{
    FileIconViewItem *item = ( FileIconViewItem* )i;

    if ( item->type() == FileIconViewItem::Dir ) {
	viewDir = QDir( item->filename() );
	readDir( viewDir );
    } else if ( item->type() == FileIconViewItem::Link &&
		QFileInfo( QFileInfo( item->filename() ).readLink() ).isDir() ) {
	viewDir = QDir( QFileInfo( item->filename() ).readLink() );
	readDir( viewDir );
    }
}

QDragObject *FileIconView::dragObject()
{
    if ( !currentItem() )
	return 0;

    QPoint orig = viewportToContents( viewport()->mapFromGlobal( QCursor::pos() ) );
    FileIconDrag *drag = new FileIconDrag( viewport() );
    drag->setPixmap( *currentItem()->pixmap(),
		     QPoint( currentItem()->pixmapRect().width() / 2, 
			     currentItem()->pixmapRect().height() / 2 ) );
    for ( FileIconViewItem *item = (FileIconViewItem*)firstItem(); item;
	  item = (FileIconViewItem*)item->nextItem() ) {
	if ( item->isSelected() ) {
	    QIconDragItem id;
	    id.setData( QCString( item->filename() ) );
	    drag->append( id,
			  QRect( item->pixmapRect( FALSE ).x() - orig.x(),
				 item->pixmapRect( FALSE ).y() - orig.y(),
				 item->pixmapRect().width(), item->pixmapRect().height() ),
			  QRect( item->textRect( FALSE ).x() - orig.x(),
				 item->textRect( FALSE ).y() - orig.y(),
				 item->textRect().width(), item->textRect().height() ),
			  QString( item->filename() ) );
	}
    }

    return drag;
}

void FileIconView::keyPressEvent( QKeyEvent *e )
{
    if ( e->key() == Key_N && ( e->state() & ControlButton ) )
	newDirectory();
    else if (e->key() == Key_X && (e->state() && ControlButton ) ||
	     e->key() == Key_Delete) 
        remove();
    else
	QIconView::keyPressEvent( e );
}

void FileIconView::slotDropped( QDropEvent *e, const QValueList<QIconDragItem> & )
{
    if ( openItem )
	openItem->timer.stop();
    if ( !QUriDrag::canDecode( e ) ) {
	e->ignore();
	return;
    }

    QStrList filelst;
    QUriDrag::decode( e, filelst );

    if ( e->action() == QDropEvent::Copy )
      copy(filelst, viewDir.absPath());
    if ( e->action() == QDropEvent::Move)
      move(filelst, viewDir.absPath());
	
    e->acceptAction();
}

void FileIconView::viewBottom()
{
    setItemTextPos( Bottom );
}

void FileIconView::viewRight()
{
    setItemTextPos( Right );
}

void FileIconView::flowEast()
{
    setHScrollBarMode( AlwaysOff );
    setVScrollBarMode( Auto );
    setArrangement( LeftToRight );
}

void FileIconView::flowSouth()
{
    setVScrollBarMode( AlwaysOff );
    setHScrollBarMode( Auto );
    setArrangement( TopToBottom );
}

void FileIconView::sortAscending()
{
    sort( TRUE );
}

void FileIconView::sortDescending()
{
    sort( FALSE );
}

void FileIconView::itemTextTruncate()
{
    setWordWrapIconText( FALSE );
}

void FileIconView::itemTextWordWrap()
{
    setWordWrapIconText( TRUE );
}

void FileIconView::mouseButton( int button, QIconViewItem *item) {
    if (button == Qt::MidButton)
        rename( item );
}

void FileIconView::contextMenu( QIconViewItem* item) {
    if ( item )
        return;
    
    // on viewport 
    QPopupMenu menu( this );
    
    menu.insertItem( "Text at the &bottom", this, SLOT( viewBottom() ) );
    menu.insertItem( "Text at the &right", this, SLOT( viewRight() ) );
    menu.insertSeparator();
    menu.insertItem( "Arrange l&eft to right", this, SLOT( flowEast() ) );
    menu.insertItem( "Arrange t&op to bottom", this, SLOT( flowSouth() ) );
    menu.insertSeparator();
    menu.insertItem( "&Truncate item text", this, SLOT( itemTextTruncate() ) );
    menu.insertItem( "&Wordwrap item text", this, SLOT( itemTextWordWrap() ) );
    menu.insertSeparator();
    menu.insertItem( "Arrange items in &grid", this, SLOT( arrangeItemsInGrid() ) );
    menu.insertSeparator();
    menu.insertItem( "Sort &ascending", this, SLOT( sortAscending() ) );
    menu.insertItem( "Sort &descending", this, SLOT( sortDescending() ) );
    
    menu.setMouseTracking( TRUE );
    menu.exec( QCursor::pos() );

}

void FileIconView::rename (QIconViewItem* item) {
    if ( item && item->renameEnabled() )
        item->rename();
}

// WARNING
// the functions below do not work if filenames 
// contains blanks ' ' characters.  Also, they are 
// currently provided only for Unix-like systems.
  
void FileIconView::remove() {
    QString message("Remove\n");
#if defined(Q_OS_UNIX)
    QString command("rm -R");
#endif

    int selection = FALSE ;
    for ( FileIconViewItem *item = (FileIconViewItem*)firstItem(); item;
	  item = (FileIconViewItem*)item->nextItem() ) {
        if ( item->isSelected() ) {
	    message += QString("%1\n").arg(item->filename());
	    command += QString(" %1").arg(item->filename());
	    selection = TRUE;
	}
    }

    if (selection && 
	QMessageBox::information(this, "Remove file(s)", message,
				 QMessageBox::Yes, QMessageBox::No, 
				 QMessageBox::Cancel) == QMessageBox::Yes)
        emit runCommand(command);
}

void FileIconView::copy(QStrList& filelst, QString destination) {
    QString message("Copy\n");
#if defined(Q_OS_UNIX)
    QString command("cp -R ");
#endif

    for ( uint i = 0; i < filelst.count(); ++i ) { 
        message += QString("%1\n").arg(filelst.at(i));
        command += QString("%1 ").arg(filelst.at(i));
    }
    message += QString("To\n%1").arg(destination);
    command += QString("%1").arg(destination);

    if (QMessageBox::information(this, "Copy file(s)", message,
				 QMessageBox::Yes, QMessageBox::No, 
				 QMessageBox::Cancel) == QMessageBox::Yes)
        emit runCommand(command);
}


void FileIconView::move(QStrList& filelst, QString destination) {
    QString message("Move\n");
#if defined(Q_OS_UNIX)
    QString command("mv ");
#endif

    for ( uint i = 0; i < filelst.count(); ++i ) { 
        message += QString("%1\n").arg(filelst.at(i));
        command += QString("%1 ").arg(filelst.at(i));
    }
    message += QString("To\n%1").arg(destination);
    command += QString("%1").arg(destination);

    if (QMessageBox::information(this, "Move file(s)", message,
				 QMessageBox::Yes, QMessageBox::No, 
				 QMessageBox::Cancel) == QMessageBox::Yes)
      emit runCommand(command);
}
