
#include "taskmanager.h"

#include <qprogressbar.h>
#include <qlabel.h>
#include <qstatusbar.h>
#include <qcombobox.h>
#include <qpixmap.h>
#include <qtoolbutton.h>
#include <qtextedit.h>
#include <qmessagebox.h>
#include <qlayout.h>

static const char*stop_xpm[]={
"22 22 10 1",
"c c #ff6363",
"e c #ff6300",
"g c #000000",
"f c #ff0031",
"# c #9c0000",
"  c None",
"b c #ff9c31",
"d c #ce0000",
"h c #ffffff",
"a c #ce6363",
"                      ",
"                      ",
"                      ",
"         #####        ",
"       ##abbcd##      ",
"      #abbeefffdg     ",
"     #cbeeffffffdg    ",
"    #abeeffffffff#g   ",
"    #beehhfffhhffdg   ",
"   #abefhhhfhhhffd#g  ",
"   #cbeffhhhhhfffd#g  ",
"   #ceffffhhhfffdd#g  ",
"   #cefffhhhhhffdd#g  ",
"   #ceffhhhfhhhdd##g  ",
"    #eefhhfffhhdd#g   ",
"    #deffffffddd##g   ",
"     #dfffffddd##g    ",
"      gdddddd###g     ",
"       gg#####gg      ",
"         ggggg        ",
"                      ",
"                      "};

static const char*clear_xpm[]={
"22 22 7 1",
"d c #319cff",
"a c #3163ff",
"# c #000000",
"c c #31ceff",
"e c #9cffff",
"  c None",
"b c #ceffff",
"                      ",
"                      ",
"                      ",
"    #############     ",
"    #############     ",
"          #           ",
"         ab#          ",
"        abcd#         ",
"       abeccd#        ",
"      abeecddd#       ",
"     ####ecd####      ",
"        abcd#         ",
"        aecd#         ",
"        aecd#         ",
"        abcd#         ",
"        aecd#         ",
"        addd#         ",
"        #####         ",
"                      ",
"                      ",
"                      ",
"                      "};

TaskManager::TaskManager( QWidget* parent, const char* name, WFlags f) 
  : QWidget ( parent, name, f )
{
    QVBoxLayout* vbox = new QVBoxLayout(this, 2, 2);

    // toolbar
    QHBoxLayout* hbox = new QHBoxLayout(2);
    vbox->addLayout( hbox, AlignRight );

    QLabel* _task_ = new QLabel( tr( " Task: " ), this );
    hbox->addWidget( _task_ );

    taskCombo = new QComboBox( TRUE, this );
    taskCombo->setAutoCompletion( TRUE );
    connect( taskCombo, SIGNAL( activated( const QString & ) ),
             this, SLOT ( runTask( const QString & ) ) );
    hbox->addWidget(taskCombo, 1);

    QPixmap pix;

    pix = QPixmap( stop_xpm );
    stopButton = new QToolButton( this, "stop");
    stopButton->setPixmap( pix );
    connect (stopButton, SIGNAL( clicked()), 
             this, SLOT( stopTask()) );
    stopButton->setEnabled( FALSE );
    hbox->addWidget(stopButton);

    pix = QPixmap( clear_xpm );
    clearButton = new QToolButton( this, "clear");
    clearButton->setPixmap( pix );
    connect (clearButton, SIGNAL( clicked()), 
             this, SLOT( clearLog()) );
    hbox->addWidget(clearButton);

    // log
    log = new QTextEdit( this, "log" );
    log->setFont(QFont("Fixed", 10, QFont::Normal));
    log->setReadOnly( TRUE );
    log->setWordWrap( QTextEdit::NoWrap );
    log->setMinimumWidth( 500 );
    vbox->addWidget(log);

    // status bar
    QStatusBar* statusBar = new QStatusBar( this );
    vbox->addWidget( statusBar );

    label = new QLabel( statusBar );
    statusBar->addWidget( label, 2, TRUE );
    progress = new QProgressBar( statusBar );
    statusBar->addWidget( progress, 1, TRUE );
    
    label->setText(" Idle");

    // process
    process = new QProcess(this);
    connect( process, SIGNAL(readyReadStderr()), 
	     this, SLOT(readFromStderr()));
    connect( process, SIGNAL(readyReadStdout()), 
	     this, SLOT(readFromStdout()));
    connect( process, SIGNAL( processExited() ),
             this, SLOT( processExited() ) );

}

void TaskManager::directoryChanged( const QString& path ) {
    QDir dir( path );
    if (dir.exists() && dir.isReadable())
        process->setWorkingDirectory( dir );
}

void TaskManager::runTask( const QString& task ) {

    if ( process->isRunning() ) {
        QMessageBox::critical(this, "Run task", 
			      "The task manager is busy!", "Ok");
	return;
    }

    process->clearArguments();
    process->setArguments( QStringList::split(' ', task) );

    if ( !process->start() ) {
        QMessageBox::critical(this, "Run task",
			      QString("task\n%1\nfailed!").arg(task),
			      "Ok");
	return;
    }

    stopButton->setEnabled( TRUE );
    clearButton->setEnabled( FALSE ); 

    setTaskCombo( task );
    log->append( QString("<b>{%1}<\b>").arg( process->workingDirectory().path() ) );
    log->append( QString("<b>%2</b>").arg( task ));
    log->append( "\n" );

    label->setText("Running...");
}

void TaskManager::setTaskCombo( const QString& task ) {
    int i = 0;
    bool found = FALSE;
    for ( i = 0; i < taskCombo->count(); ++i ) {
        if ( taskCombo->text( i ) == task) {
            found = TRUE;
            break;
        }
    }

    if ( found )
        taskCombo->setCurrentItem( i );
    else {
        taskCombo->insertItem( task );
        taskCombo->setCurrentItem( taskCombo->count() - 1 );
    }
 
}

void TaskManager::stopTask() {
    if ( process->isRunning() )
        process->kill();
    log->append( "<b>killed<\b>" );
    log->append( "\n" );
}

void TaskManager::clearLog() {
    log->setText("");
}

void TaskManager::readFromStderr() {
    log->append( process->readStderr() );
}

void TaskManager::readFromStdout() {
    log->append( process->readStdout() );
}

void TaskManager::processExited() {
    stopButton->setEnabled( FALSE );
    clearButton->setEnabled( TRUE );
    log->append("\n\n");
    label->setText(" Idle");
}
