
#include "toolsetup.h"

#include <qlabel.h>
#include <qhbox.h>

void ToolSetup::setup() {
  QVBoxLayout* vbox = new QVBoxLayout(this, 6, 6);
  fill( vbox );

  // ok & cancel
  QHBoxLayout* hbox = new QHBoxLayout(6);
  vbox->addLayout(hbox, AlignRight);
  
  QPushButton* ok = new QPushButton( "Ok", this);
  ok->setDefault( TRUE );
  hbox->addWidget( ok );
  
  QPushButton* cancel = new QPushButton( "Cancel", this); 
  hbox->addWidget( cancel );
  
  QSize bs = cancel->sizeHint(); // cancel is larger
  ok->setFixedSize(bs);
  cancel->setFixedSize(bs);
  
  connect(ok, SIGNAL(clicked()), 
	  this, SLOT(accept()));
  connect(cancel, SIGNAL(clicked()), 
	  this, SLOT(reject()));

  resize( 480, 0 );
}

void ToolSetup::fill( QVBoxLayout* ) {
  // virtual 
}

void ToolSetup::addBoolOption( QVButtonGroup* bg, 
			       BoolOption* bo, const char* name) {
  bo->cb = new QCheckBox( name, bg ); 
  bo->cb->setChecked( FALSE );
}

void ToolSetup::addFileOption( QVButtonGroup* bg,
			       FileOption* fo, const char* name) {
  fo->cb = new QCheckBox( name, bg );
  QHBox* hbox = new QHBox( bg );
  (void) new QLabel( "   ", hbox );
  fo->ed = new QLineEdit( "", hbox );
  fo->br = new QPushButton( "Browse...", hbox );

  fo->cb->setChecked( FALSE );
  fo->ed->setEnabled( FALSE );
  fo->br->setEnabled( FALSE );
}

void ToolSetup::addIntegerOption( QVButtonGroup* bg,
				  IntegerOption* io, const char* name,
				  const int min, const int max, const int step) {

  io->cb = new QCheckBox( name, bg);
  QHBox* hbox = new QHBox( bg );
  (void) new QLabel( "   ", hbox );
  io->sb = new QSpinBox( min, max, step, hbox );

  io->cb->setChecked( FALSE );
  io->sb->setEnabled( FALSE ); 
}

/*
 *
 */

void GeneratorSetup::fill( QVBoxLayout* vbox )  {
    QLabel* file = new QLabel("Generate " + filename, this);
    vbox->addWidget( file );
 
    // traversal 

    QVButtonGroup* bg1 = new QVButtonGroup( this );
    bg1->setTitle("Traversal");
    bg1->setRadioButtonExclusive(TRUE);
    vbox->addWidget( bg1 );

    QRadioButton* bfs = new QRadioButton("Breadth First Search", bg1);
    connect(bfs, SIGNAL(clicked()), 
	    this, SLOT(clickBfs()));
    QRadioButton* dfs = new QRadioButton("Depth First Search", bg1);
    connect(dfs, SIGNAL(clicked()), 
	    this, SLOT(clickDfs()));
    
    traversal = BFS;
    bfs->setChecked( TRUE );
    
    // options
    
    QVButtonGroup* bg2 = new QVButtonGroup( this );
    bg2->setTitle("Options");
    vbox->addWidget( bg2 );
    
    addBoolOption( bg2, &partial, "Partial Order Reduction" );
    addBoolOption( bg2, &split, "Split Multi-Event Transitions" );

    partial.cb->setEnabled( FALSE );
    
    // output

    QVButtonGroup* bg3 = new QVButtonGroup( this );
    bg3->setTitle("Output");
    vbox->addWidget( bg3);

    addFileOption(bg3, &states, "States File");
    connect( states.cb, SIGNAL(clicked()), 
	     this, SLOT(clickOutputStates()));
    connect( states.br, SIGNAL(clicked()), 
	     this, SLOT(clickBrowseStates()));

    addFileOption(bg3, &transitions, "Transitions File");
    connect( transitions.cb, SIGNAL(clicked()), 
	     this, SLOT(clickOutputTransitions()));
    connect( transitions.br, SIGNAL(clicked()), 
	     this, SLOT(clickBrowseTransitions()));
}

QString GeneratorSetup::command() {
  QString command(filename.replace(filename.length() - 2, 2, "x"));

  switch (traversal) {
  case BFS: command += " -bfs"; break;
  case DFS: command += " -dfs"; break;
  } 
  if (traversal == DFS && partial.isOn())
    command += " -po";
  if (split.isOn())
    command += " -sp";
  if (states.isOn()) 
    command += QString(" -q %1").arg( states.filename() );
  if (transitions.isOn()) 
    command += QString(" -t %1").arg( transitions.filename() );

  return command;
}

/*
 *
 */

void OptimizerSetup::fill( QVBoxLayout* vbox ) {
    QLabel* file = new QLabel("Optimize " + filename, this);
    vbox->addWidget( file );

    // analysis

    QVButtonGroup* bg1 = new QVButtonGroup( this );
    bg1->setTitle("Analysis");
    bg1->setRadioButtonExclusive(TRUE);
    vbox->addWidget( bg1 );

    QRadioButton* live = new QRadioButton("Live variables reduction", bg1);
    connect(live, SIGNAL(clicked()), 
	    this, SLOT(clickLive()));
    QRadioButton* reach = new QRadioButton("Dead code elimination", bg1);
    connect(reach, SIGNAL(clicked()), 
	    this, SLOT(clickReach()));
    QRadioButton* slice = new QRadioButton("Slice variables", bg1);
    connect(slice, SIGNAL(clicked()), 
	    this, SLOT(clickSlice()));
    
    analysis = LIVE;
    live->setChecked( TRUE );

    // options

    QVButtonGroup* bg2 = new QVButtonGroup( this );
    bg2->setTitle("Options");
    vbox->addWidget( bg2 );

    addFileOption (bg2, &configure, "Configure File");
    connect(configure.cb, SIGNAL(clicked()), 
	    this, SLOT(clickOptionConfigure()));
    connect(configure.br, SIGNAL(clicked()), 
	    this, SLOT(clickBrowseConfigure()));
    
    // output 

    QVButtonGroup* bg3 = new QVButtonGroup( this );
    bg3->setTitle("Output");
    vbox->addWidget( bg3 );

    addFileOption (bg3, &result, "Result File");
    connect(result.cb, SIGNAL(clicked()), 
	    this, SLOT(clickOutputResult()));
    connect(result.br, SIGNAL(clicked()), 
	    this, SLOT(clickBrowseResult()));
}

QString OptimizerSetup::command() {
  QString command = "dfa";
  switch (analysis) {
  case LIVE: command += " -live"; break;
  case REACH: command += " -reach"; break;
  case SLICE: command += " -slice"; break;
  }
  if (configure.isOn()) 
    command += QString(" -c %1").arg(configure.filename());
  if (result.isOn()) 
    command += QString(" -o %1").arg(result.filename());

  command += QString(" %1").arg(filename);

  return command;
}

/*
 *
 */

void AldebaranSetup::fill( QVBoxLayout* vbox ) {
  QLabel* file = new QLabel("Minimize " + filename, this);
  vbox->addWidget( file );

  // analysis
  
  QVButtonGroup* bg1 = new QVButtonGroup( this );
  bg1->setTitle("Bisimulation");
  bg1->setRadioButtonExclusive(TRUE);
  vbox->addWidget( bg1 );

  QRadioButton* bmin = new QRadioButton("Strong Bisimulation", bg1);
  connect(bmin, SIGNAL(clicked()), 
	  this, SLOT(clickBmin()));
  QRadioButton* pmin = new QRadioButton("Branching Bisimulation", bg1);
  connect(pmin, SIGNAL(clicked()), 
	  this, SLOT(clickPmin()));
  QRadioButton* omin = new QRadioButton("Observational Bisimulation", bg1);
  connect(omin, SIGNAL(clicked()), 
	  this, SLOT(clickOmin()));
  QRadioButton* imin = new QRadioButton("tau*a Bisimulation", bg1);
  connect(imin, SIGNAL(clicked()), 
	  this, SLOT(clickImin()));
  QRadioButton* smin = new QRadioButton("Safety Bisimulation", bg1);
  connect(smin, SIGNAL(clicked()), 
	  this, SLOT(clickSmin()));
  
  bisimulation = BMIN;
  bmin->setChecked( TRUE );
  
  // hide & rename

  QVButtonGroup* bg2 = new QVButtonGroup( this );
  bg2->setTitle("Options");
  vbox->addWidget( bg2 );

  addFileOption( bg2, &hide, "Hide File");
  connect(hide.cb, SIGNAL(clicked()), 
	  this, SLOT(clickOptionHide()));  
  connect(hide.br, SIGNAL(clicked()), 
	  this, SLOT(clickBrowseHide()));

  addFileOption( bg2, &rename, "Rename File");
  connect(rename.cb, SIGNAL(clicked()), 
	  this, SLOT(clickOptionRename()));  
  connect(rename.br, SIGNAL(clicked()), 
	  this, SLOT(clickBrowseRename()));
  
  // output 

  QVButtonGroup* bg3 = new QVButtonGroup( this );
  bg3->setTitle("Output");
  vbox->addWidget( bg3 );
  
  addFileOption( bg3, &result, "Result File");
  connect(result.cb, SIGNAL(clicked()), 
	  this, SLOT(clickOutputResult()));  
  connect(result.br, SIGNAL(clicked()), 
	  this, SLOT(clickBrowseResult()));
}

QString AldebaranSetup::command() {
  QString command("aldebaran");
  switch (bisimulation) {
  case BMIN: command += " -bmin"; break;
  case PMIN: command += " -pmin"; break;
  case OMIN: command += " -omin"; break;
  case IMIN: command += " -imin"; break;
  case SMIN: command += " -smin"; break;
  }
  if (hide.isOn()) 
    command += QString(" -hide %1").arg( hide.filename() );
  if (rename.isOn()) 
    command += QString(" -rename %1").arg( rename.filename() );
  if (result.isOn()) 
    command += QString(" -output %1").arg( result.filename() );
  command += QString(" %1").arg( filename );

  return command;
}

/* 
 *
 */

void LashSetup::fill( QVBoxLayout* vbox ) {
  // 
  QLabel* file = new QLabel("Analyze " + filename, this);
  vbox->addWidget( file );

  // options

  QVButtonGroup* bg2 = new QVButtonGroup( this );
  bg2->setTitle("Options");
  vbox->addWidget( bg2 );

  addBoolOption( bg2, &verbose, "Verbose Output" );
  addBoolOption( bg2, &explore, "State Space Exploration");

  // output

  QVButtonGroup* bg3 = new QVButtonGroup( this );
  bg3->setTitle("Output");
  vbox->addWidget( bg3 );

  addFileOption( bg3, &sm, "State Machine");
  connect(sm.cb, SIGNAL(clicked()), 
	  this, SLOT(clickOutputSm()));  
  connect(sm.br, SIGNAL(clicked()), 
	  this, SLOT(clickBrowseSm()));

  addFileOption( bg3, &ndd, "NDD File");
  connect(ndd.cb, SIGNAL(clicked()), 
	  this, SLOT(clickOutputNdd()));  
  connect(ndd.br, SIGNAL(clicked()), 
	  this, SLOT(clickBrowseNdd()));

  addFileOption( bg3, &dot, "DOT File");
  connect(dot.cb, SIGNAL(clicked()), 
	  this, SLOT(clickOutputDot()));  
  connect(dot.br, SIGNAL(clicked()), 
	  this, SLOT(clickBrowseDot()));

}

QString LashSetup::command() {
  QString command("siflash");
  if (verbose.isOn())
    command += " -v";
  if (!explore.isOn())
    command += " -c";
  if (sm.isOn()) 
    command += QString(" -m %1").arg(sm.filename());
  if (ndd.isOn()) 
    command += QString(" -ndd %1").arg(ndd.filename());
  if (dot.isOn()) 
    command += QString(" -dot %1").arg(dot.filename());
  command += QString(" %1").arg(filename);

  return command;
}

/*
 *
 */

void TrexSetup::fill( QVBoxLayout* vbox ) {
  // 
  QLabel* file = new QLabel("Analyze " + filename, this);
  vbox->addWidget( file );

  QHBoxLayout* hbox = new QHBoxLayout(6);
  vbox->addLayout(hbox, AlignRight);

  QWidget* l = new QWidget( this, "l-pane" );
  QVBoxLayout* l_vbox = new QVBoxLayout( l, 3, 3);
  hbox->addWidget( l );
  QWidget* r = new QWidget( this, "r-pane" );
  QVBoxLayout* r_vbox = new QVBoxLayout( r, 3, 3);
  hbox->addWidget( r );

  QVButtonGroup* bg1 = new QVButtonGroup( l );
  bg1->setTitle("Analysis");
  bg1->setRadioButtonExclusive( TRUE );
  l_vbox->addWidget( bg1 );

  // analysis { forward / backward }

  QRadioButton* forward = new QRadioButton("Forward", bg1);
  connect( forward, SIGNAL( clicked() ), 
	   this, SLOT( clickForward() ));
  QRadioButton* backward = new QRadioButton("Backward", bg1);
  connect( backward, SIGNAL( clicked() ),
	   this, SLOT( clickBackward() ));
  
  analysis = FORWARD;
  forward->setChecked( TRUE );

  // solver { reduce, foaf }

  QVButtonGroup* bg2 = new QVButtonGroup( r );
  bg2->setTitle("Solver");
  bg2->setRadioButtonExclusive( TRUE );
  r_vbox->addWidget( bg2 );
  
  QRadioButton* reduce = new QRadioButton( "Reduce", bg2);
  connect( reduce, SIGNAL( clicked() ),
	   this, SLOT( clickReduce() ));
  QRadioButton* foaf = new QRadioButton( "Foaf", bg2);
  connect( foaf, SIGNAL( clicked() ),
	   this, SLOT( clickFoaf() ));
    
  solver = REDUCE;
  reduce->setChecked( TRUE );

  // initial 

  QVButtonGroup* bg3 = new QVButtonGroup( l );
  bg3->setTitle("Initial");
  l_vbox->addWidget( bg3 );
  
  addFileOption (bg3, &initial.constraint, "Constraint");
  connect(initial.constraint.cb, SIGNAL(clicked()), 
	  this, SLOT(clickInitialConstraint()));
  connect(initial.constraint.br, SIGNAL(clicked()), 
	  this, SLOT(clickBrowseInitialConstraint()));
  
  addFileOption( bg3, &initial.configuration, "Configuration");
  connect(initial.configuration.cb, SIGNAL( clicked() ),
	  this, SLOT(clickInitialConfiguration()));
  connect(initial.configuration.br, SIGNAL(clicked()),
	  this, SLOT(clickBrowseInitialConfiguration()));

  // extrapolation

  // observer

  QVButtonGroup* bg5 = new QVButtonGroup( r );
  bg5->setTitle( "Observer" );
  r_vbox->addWidget( bg5 );

  addFileOption(bg5, &observer, "Observer");
  connect( observer.cb, SIGNAL( clicked()),
	   this, SLOT( clickObserver() ));
  connect( observer.br, SIGNAL( clicked()),
	   this, SLOT( clickBrowseObserver() ));

  // extrapolation

  QVButtonGroup* bg4 = new QVButtonGroup( r );
  bg4->setTitle( "Extrapolation" );
  r_vbox->addWidget( bg4 );

  addBoolOption( bg4, &extrapolation.substitute, "Substitute Variables");
  
  addFileOption( bg4, &extrapolation.states, "States");
  connect( extrapolation.states.cb, SIGNAL( clicked() ),
	   this, SLOT( clickExtrapolationStates() ));
  connect( extrapolation.states.br, SIGNAL( clicked() ),
	   this, SLOT( clickBrowseExtrapolationStates() ));

  addIntegerOption(bg4, &extrapolation.depth, "Depth", 0, 10, 1); 
  connect ( extrapolation.depth.cb, SIGNAL( clicked()),
	    this, SLOT( clickExtrapolationDepth() ));


  // options 

  QVButtonGroup* bg6 = new QVButtonGroup( r );
  bg6->setTitle( "Options" );
  r_vbox->addWidget( bg6 );

  addBoolOption( bg6, &option.statistics, "Produce Statistics");
  addBoolOption( bg6, &option.showOperations, "Show All Label Operations");
  addBoolOption( bg6, &option.simplification, "Systematic Simplifications");
  addBoolOption( bg6, &option.counterSet, "Consider Counters as Set");
  addIntegerOption( bg6, &option.reduceCalls, "Maximal Reduce Calls", 0, 100, 10); 
  connect( option.reduceCalls.cb, SIGNAL( clicked()),
	   this, SLOT( clickOptionReduceCalls() ));

  // output
  
  QVButtonGroup * bg7 = new QVButtonGroup( l );
  bg7->setTitle( "Output" );
  l_vbox->addWidget( bg7 );
  
  addFileOption( bg7, &output.configurations, "Configurations");
  connect(output.configurations.cb, SIGNAL( clicked()),
	  this, SLOT( clickOutputConfigurations()) );
  connect(output.configurations.br, SIGNAL( clicked() ),
	  this, SLOT( clickBrowseOutputConfigurations()));
  
  addFileOption( bg7, &output.finiteGraph_1, "Finite Graph / 1st form");
  connect(output.finiteGraph_1.cb, SIGNAL( clicked()),
	  this, SLOT( clickOutputFiniteGraph_1()) );
  connect(output.finiteGraph_1.br, SIGNAL( clicked() ),
	  this, SLOT( clickBrowseOutputFiniteGraph_1()));

  addFileOption( bg7, &output.finiteGraph_2, "Finite Graph / 2nd form");
  connect(output.finiteGraph_2.cb, SIGNAL( clicked()),
	  this, SLOT( clickOutputFiniteGraph_2()) );
  connect(output.finiteGraph_2.br, SIGNAL( clicked() ),
	  this, SLOT( clickBrowseOutputFiniteGraph_2()));
  
  addFileOption( bg7, &output.trace, "Analysis Trace");
  connect(output.trace.cb, SIGNAL( clicked()),
	  this, SLOT( clickOutputTrace()) );
  connect(output.trace.br, SIGNAL( clicked() ),
	  this, SLOT( clickBrowseOutputTrace()));


} 

QString TrexSetup::command() {
  QString command("trex ");
  command += QString(" -init %1").arg(filename);

  if (analysis == BACKWARD)
    command += " -pre";
  if (solver == FOAF)
    command += " -e";

  if (initial.constraint.isOn()) 
    command += QString(" -c %1").arg( initial.constraint.filename() );
  if (initial.configuration.isOn())
    command += QString(" -cf %1").arg( initial.configuration.filename() );

  if (observer.isOn())
    command += QString(" -prop %1").arg( observer.filename() );

  if (extrapolation.substitute.isOn())
    command += " -ne";
  if (extrapolation.states.isOn())
    command += QString(" -a %1").arg( extrapolation.states.filename() );
  if (extrapolation.depth.isOn())
    command += QString(" -cp %1").arg( extrapolation.depth.value() );

  if (option.statistics.isOn())
    command += " -stat";
  if (option.showOperations.isOn())
    command += " -env";
  if (option.simplification.isOn())
    command += " -simpl";
  if (option.counterSet.isOn() )
    command += " -set";
  if (option.reduceCalls.isOn() )
    command += QString(" -red %1").arg( option.reduceCalls.value() );
  
  if (output.configurations.isOn())
    command += QString(" -res %1").arg( output.configurations.filename() );
  if (output.finiteGraph_1.isOn())
    command += QString(" -sg %1").arg( output.finiteGraph_1.filename() );
  if (output.finiteGraph_2.isOn())
    command += QString(" -sg2 %1").arg( output.finiteGraph_2.filename() );
  if (output.trace.isOn())
    command += QString(" -tr %1").arg( output.trace.filename() );
  
  return command;
}
