
#ifndef TOOLSETUP_H
#define TOOLSETUP_H

#include <qlayout.h>
#include <qvbuttongroup.h>
#include <qdialog.h>
#include <qfiledialog.h>

#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qlineedit.h>
#include <qspinbox.h>

class ToolSetup : public QDialog {
  Q_OBJECT

public:
  ToolSetup( QWidget* parent, const char* name) 
    : QDialog( parent, name ) 
    { setCaption(QString("%1 setup").arg(name)); }

  virtual QString command() 
    { return QString::null; }

  void setup();

protected:
  virtual void fill( QVBoxLayout* vbox );

protected:
  struct BoolOption {
    QCheckBox* cb;
    
    inline bool isOn() const 
      { return cb->isOn(); } 
  };

  struct FileOption {
    QCheckBox* cb;
    QLineEdit* ed;
    QPushButton* br;
    
    inline bool isOn() const 
      { return cb->isOn(); }
    inline QString filename() const 
      { return ed->text(); }
    inline void update() 
      { ed->setEnabled(isOn()); 
        br->setEnabled(isOn()); }
    inline void browse(ToolSetup* w, const char* filemask)
      { QString fn = QFileDialog::getOpenFileName( QString::null, filemask, w );
        ed->setText( fn ); }
  };

  struct IntegerOption {
    QCheckBox* cb;
    QSpinBox* sb;

    inline bool isOn() const
      { return cb->isOn(); }
    inline int value() const
      { return sb->value(); }
    inline void update() 
      { sb->setEnabled(isOn()); }
  };
  
  virtual void addBoolOption( QVButtonGroup* bg,
			      BoolOption* bo, const char* name );
  virtual void addFileOption( QVButtonGroup* bg, 
			      FileOption* fo, const char* name );
  virtual void addIntegerOption( QVButtonGroup* bg,
				 IntegerOption* io, const char* name,
				 const int min, const int max, const int step);

};

/*
 *
 */

class GeneratorSetup : public ToolSetup {
  Q_OBJECT 

public:
  GeneratorSetup( QWidget* parent, QString& fn ) 
    : ToolSetup( parent, "generator"), filename(fn) 
    { }

  virtual QString command(); 
  
protected:
  virtual void fill( QVBoxLayout* vbox );

protected:
  QString filename;

  enum Traversal {BFS, DFS} traversal; 

  BoolOption partial;
  BoolOption split;
  FileOption states;
  FileOption transitions;
  
protected slots:
  void clickBfs() { traversal = BFS; partial.cb->setEnabled(FALSE); }
  void clickDfs() { traversal = DFS; partial.cb->setEnabled(TRUE); }
  void clickOutputStates() { states.update(); }
  void clickBrowseStates() { states.browse( this, "States (*.states)"); }
  void clickOutputTransitions() { transitions.update(); }
  void clickBrowseTransitions() { transitions.browse( this, "Automata (*.aut)"); }
  
};

/*
 *
 */

class OptimizerSetup : public ToolSetup {
  Q_OBJECT

public:
  OptimizerSetup( QWidget* parent, QString& fn ) 
    : ToolSetup( parent, "optimizer"), filename (fn) 
    { }
  
  virtual QString command();

protected:
  virtual void fill ( QVBoxLayout* vbox );

protected:
  QString filename;

  enum Analysis { LIVE, REACH, SLICE } analysis;
  
  FileOption configure;
  FileOption result;
 
protected slots:
  void clickLive() { analysis = LIVE; }
  void clickReach() { analysis = REACH; }
  void clickSlice() { analysis = SLICE; }
  void clickOptionConfigure() { configure.update(); }
  void clickBrowseConfigure() { configure.browse( this, "Configure (*.cfg)"); }
  void clickOutputResult() { result.update(); }
  void clickBrowseResult() { result.browse( this, "IF Files (*.if)"); };

};

/*
 *
 */

class AldebaranSetup : public ToolSetup {
  Q_OBJECT

public:
  AldebaranSetup( QWidget* parent, QString& fn)
    : ToolSetup( parent, "aldebaran"), filename( fn ) 
    { }
  
  virtual QString command();

protected:
  virtual void fill ( QVBoxLayout* vbox);

protected: 
  QString filename;
  
  enum Bisimulation 
    { BMIN, PMIN, OMIN, SMIN, IMIN} bisimulation;

  FileOption hide;
  FileOption rename;
  FileOption result;

protected slots:
  void clickBmin() { bisimulation = BMIN; }
  void clickPmin() { bisimulation = PMIN; } 
  void clickSmin() { bisimulation = SMIN; }
  void clickImin() { bisimulation = IMIN; }
  void clickOmin() { bisimulation = OMIN; }
  
  void clickOptionHide() { hide.update(); }
  void clickBrowseHide() { hide.browse( this, "Hiding rules (*.hide)"); }
  void clickOptionRename() { rename.update(); }
  void clickBrowseRename() { rename.browse( this, "Renaming rules (*.rename)"); }
  
  void clickOutputResult() { result.update(); }
  void clickBrowseResult() { result.browse( this, "Automata (*.aut)"); }

};

/*
 *
 */

class LashSetup : public ToolSetup {
  Q_OBJECT

public:
  LashSetup( QWidget* parent, QString& fn)
    : ToolSetup( parent, "lash"), filename( fn ) 
    { }
  
  virtual QString command();

protected:
  virtual void fill ( QVBoxLayout* vbox);
 
protected: 
  QString filename;

  BoolOption verbose;
  BoolOption explore;
  FileOption sm;
  FileOption ndd;
  FileOption dot;

protected slots:
  void clickOutputSm() { sm.update(); }
  void clickBrowseSm() { sm.browse( this, "State Machines (*.lash)"); } 
  void clickOutputNdd() { ndd.update(); }
  void clickBrowseNdd() { ndd.browse( this, "Numerical Decision Diagrams (*.ndd)"); }
  void clickOutputDot() { dot.update(); }
  void clickBrowseDot() { dot.browse( this, "Dot Files (*.dot)"); }

};

/*
 *
 */

class TrexSetup : public ToolSetup {
  Q_OBJECT

public:
  TrexSetup( QWidget* parent, QString& fn )
    : ToolSetup( parent, "trex" ), filename( fn ) 
    { }

  virtual QString command();

protected:
  virtual void fill( QVBoxLayout* vbox );

protected:
  QString filename;

  enum { FORWARD, BACKWARD } analysis;
  enum { REDUCE, FOAF } solver;

  struct {
    FileOption constraint;
    FileOption configuration;
  } initial;

  struct {
    BoolOption substitute;
    FileOption states;
    IntegerOption depth;
  } extrapolation;

  FileOption observer;

  struct {
    BoolOption statistics;
    BoolOption simplification;
    BoolOption counterSet;
    BoolOption showOperations;
    IntegerOption reduceCalls;
  } option;

  struct {
    FileOption configurations;
    FileOption finiteGraph_1;
    FileOption finiteGraph_2;
    FileOption trace;
  } output;

protected slots:
  void clickForward() { analysis = FORWARD; }
  void clickBackward() { analysis = BACKWARD; }
  void clickReduce() { solver = REDUCE; }
  void clickFoaf() { solver = FOAF; }
  
  void clickInitialConstraint() { initial.constraint.update(); }
  void clickBrowseInitialConstraint() { initial.constraint.browse(this, "All files (*)"); }
  void clickInitialConfiguration() { initial.configuration.update(); }
  void clickBrowseInitialConfiguration() { initial.configuration.browse(this, "All files (*)"); }

  void clickExtrapolationStates() { extrapolation.states.update(); }
  void clickBrowseExtrapolationStates() { extrapolation.states.browse(this, "All files (*)"); }
  void clickExtrapolationDepth() { extrapolation.depth.update(); }
  
  void clickObserver() { observer.update(); }
  void clickBrowseObserver() { observer.browse(this, "All files (*)"); }

  void clickOptionReduceCalls() { option.reduceCalls.update();}

  void clickOutputConfigurations() { output.configurations.update(); }
  void clickBrowseOutputConfigurations() { output.configurations.browse( this, "All files (*)"); }
  void clickOutputFiniteGraph_1() { output.finiteGraph_1.update(); }
  void clickBrowseOutputFiniteGraph_1() { output.finiteGraph_1.browse(this, "All files (*)"); } 
  void clickOutputFiniteGraph_2() { output.finiteGraph_2.update(); }
  void clickBrowseOutputFiniteGraph_2() { output.finiteGraph_2.browse(this, "All files (*)"); } 
  void clickOutputTrace() { output.trace.update(); }
  void clickBrowseOutputTrace() { output.trace.browse( this, "All files (*)"); }

};

#endif
